

	/****************************************************
	 *	Functions to control the visualisation window
	 ****************************************************/
	
	var Vis = {};

	Vis.dirList = [];
	Vis.descList = [];
	Vis.curVis = 0;

	Vis.init = function()
	{
		Vis.getDirectoryList();
		Vis.guiEvents();
		Vis.generate();
	}

	Vis.getSpectrum = function( FFT, stretch )
	{
		var FFT = ( typeof FFT == 'undefined' ) ? true : FFT;
		var stretch = ( typeof stretch == 'undefined' ) ? 0 : stretch;
		var data = [];
		var bytes = new air.ByteArray(); 

		air.SoundMixer.computeSpectrum( bytes, FFT, stretch );

		if( bytes )
		{
	        for ( var i = 0; i < 512; i++ )  
	        { 
	            bytes.position = i * 4; 
	            n = bytes.readFloat(); 
				// Collect data
	            data.push( n ); 
	        } 
		}		
		return data; 
	}

	Vis.getPeaks = function()
	{
		return [ window.parent.CD.Audio.channel.leftPeak, window.parent.CD.Audio.channel.rightPeak ];	
	}

	/**
	 * Read directory list and open the description files
	 */
	Vis.getDirectoryList = function()
	{
		var visualDirs = air.File.applicationDirectory.resolvePath('visualisations').getDirectoryListing();
		var filestream = new air.FileStream();
		
		for ( i=0; i < visualDirs.length; i++ ) 
		{
			if( visualDirs[i].name[0] !== '.' )
			{
			    Vis.dirList.push( visualDirs[i].name );
	
				/**
				 * Open description files
				 */
				var desc = air.File.applicationDirectory.resolvePath( 'visualisations/' + visualDirs[i].name + '/desc.json' );
	
				filestream.open( desc, air.FileMode.READ );
				rsp = filestream.readUTFBytes( filestream.bytesAvailable );
	
				// Store the descriptions
				Vis.descList.push( JSON.parse( rsp ) ); 	
	
				filestream.close();
			}
		}
	}

	Vis.generate = function()
	{
	    var flashvars = {};

	    var params = {};
	    params.quality = "high";
	    params.bgcolor = "#000000";
	    params.allowscriptaccess = "sameDomain";
	
	    var attributes = {};
	    attributes.id = "visualisation_flash";

		var dir = Vis.dirList[ Vis.curVis ];
		var desc = Vis.descList[ Vis.curVis ];

		var file = 'visualisations/' + dir + '/visual.swf';
		$('description').innerHTML = desc.name + ' - <a href="' + desc.author_url + '">' + desc.author + '</a>';
	
	    swfobject.embedSWF(
	        file, "visualisation",
	        "100%", "100%",
	        "0", "",
	        flashvars, params, attributes );
	}

	Vis.guiEvents = function()
	{
		$('prev').addEvent( 'click', function( e )
		{
			e.stop();
			Vis.curVis = ( Vis.curVis == 0 ) ?  Vis.dirList.length-1 : --Vis.curVis;
			var dir = Vis.dirList[ Vis.curVis ];
			var desc = Vis.descList[ Vis.curVis ];

			$('visualisation_flash').destroy();
			new Element('div',{id:'visualisation'}).inject( document.body );
			Vis.generate();

			$('description').innerHTML = desc.name + ' - <a href="' + desc.author_url + '">' + desc.author + '</a>';
		});

		$('next').addEvent( 'click', function( e )
		{
			e.stop();
			Vis.curVis = ( Vis.curVis == Vis.dirList.length-1 ) ? 0 : ++Vis.curVis;
			var dir = Vis.dirList[ Vis.curVis ];
			var desc = Vis.descList[ Vis.curVis ];
			
			$('visualisation_flash').destroy();
			new Element('div',{id:'visualisation'}).inject( document.body );
			Vis.generate();

			$('description').innerHTML = desc.name + ' - <a href="' + desc.author_url + '">' + desc.author + '</a>';
		});
	}

	window.htmlLoader.navigateInSystemBrowser = true;

	/**
	 * Starta visualisationkurs NU!
	 */
	window.addEvent( 'load', Vis.init );

	var log = function( obj )
	{
		air.Introspector.Console.log( obj );
	}
